@@ -148,7 +148,21 @@ def signal_handler(signum, frame, verbose=False):
 def _stream_reader(args):
     unbuffer = args[0]
     stream = args[1]
-    write = tuple( x for x in args[2:] if x is not None )
+    output = tuple( x for x in args[2:] if x is not None )
+    def write(x):
+        success = True
+        for s in output:
+            try:
+                s.write(x)
+            except ValueError:
+                success = False
+        return success
+    def flush():
+        for s in output:
+            try:
+                s.flush()
+            except ValueError:
+                pass
 
     encoding = sys.__stdout__.encoding
     if encoding is None:
@@ -170,29 +184,35 @@ def _stream_reader(args):
             continue
         data = ""
         if unbuffer == 1:
-            for s in write:
-                s.write(char)
+            writeOK = write(char)
         buf += char
         if char[-1] != "\n":
             continue
         if unbuffer:
             if unbuffer != 1:
-                for s in write:
-                    s.write(buf)             
-            for s in write:
-                s.flush()
+                writeOK = write(buf)
+            flush()
         else:
-            for s in write:
-                s.write(buf)
-        buf = ""
+            writeOK = write(buf)
+        if writeOK:
+            buf = ""
+    writeOK = True
     if buf:
-        for s in write:
-            s.write(buf)
+        writeOK &= write(buf)
     if data:
-        for s in write:
-            s.write(data.decode(encoding))
-    for s in write:
-        s.flush()
+        writeOK &= write(data.decode(encoding))
+    flush()
+    if not writeOK:
+        sys.__stderr__.write("""
+ERROR: pyutilib.subprocess: output stream closed before all subprocess output
+       was written to it.  The following was left in the subprocess buffer:
+            '%s'
+""" % (buf,))
+        if data:
+            sys.__stderr__.write(
+"""The following undecoded unicode output was also present:
+            '%s'
+""" % (data,))
         
 #
 # A function used to read in data from two independent streams and push
@@ -207,16 +227,31 @@ def _stream_reader(args):
 #
 def _merged_reader(*args):
     class StreamData(object):
-        __slots__ = ( 'read','write','unbuffer','buf','data' )
+        __slots__ = ( 'read','output','unbuffer','buf','data' )
         def __init__(self, *args):
             if subprocess.mswindows:
                 self.read = get_osfhandle( args[1] )
             else:
                 self.read = args[1]
             self.unbuffer = args[0]
-            self.write = tuple( x for x in args[2:] if x is not None )
+            self.output = tuple( x for x in args[2:] if x is not None )
             self.buf = ""
             self.data = None
+
+        def write(self,x):
+            success = True
+            for s in self.output:
+                try:
+                    s.write(x)
+                except ValueError:
+                    success = False
+            return success
+        def flush(self):
+            for s in self.output:
+                try:
+                    s.flush()
+                except ValueError:
+                    pass
         
     encoding = sys.__stdout__.encoding
     if encoding is None:
@@ -232,13 +267,18 @@ def _merged_reader(*args):
 
     while handles:
         if subprocess.mswindows:
+            new_data = None
             for h in handles:
-                numAvail = PeekNamedPipe(h, 0)[1]
-                if numAvail == 0:
-                    if h.closed:
-                        handles.remove(h)
-                    continue
-                new_data = ReadFile(h, 1, None)
+                try:
+                    numAvail = PeekNamedPipe(h, 0)[1]
+                    if numAvail == 0:
+                        continue
+                    result, new_data = ReadFile(h, 1, None)
+                except:
+                    handles.remove(h)
+                    new_data = None
+            if new_data is None:
+                continue
         else:
             h = select(handles,noop,noop)[0]
             if not h:
@@ -258,28 +298,34 @@ def _merged_reader(*args):
             continue
         s.data = None
         if s.unbuffer:
-            for x in s.write:
-                x.write(char)
+            writeOK = s.write(char)
         s.buf += char
         if char[-1] != "\n":
             continue
         if s.unbuffer:
-            for x in s.write:
-                x.flush()
+            s.flush()
         else:
-            for x in s.write:
-                x.write(s.buf)
-        s.buf = ""
+            writeOK = s.write(s.buf)
+        if writeOK:
+            s.buf = ""
+    writeOK = True
     for s in itervalues(streams):
         if s.buf:
-            for x in s.write:
-                x.write(s.buf)
+            writeOK &= s.write(s.buf)
         if s.data:
-            tmp = s.data.decode(encoding)
-            for x in s.write:
-                x.write(tmp)
-        for x in s.write:
-            x.flush()
+            writeOK &= s.write(s.data.decode(encoding))
+        s.flush()
+    if not writeOK:
+        sys.__stderr__.write("""
+ERROR: pyutilib.subprocess: output stream closed before all subprocess output
+       was written to it.  The following was left in the subprocess buffer:
+            '%s'
+""" % (buf,))
+        if data:
+            sys.__stderr__.write(
+"""The following undecoded unicode output was also present:
+            '%s'
+""" % (data,))
         
 #
 # A mock-up of the _merged_reader for platforms (or installations) that
